/* FILE NAME: server.c */
#include <stdio.h>
#include "rfile.h"                     /* header created by the idl compiler */
#include "status.h"                    /* contains the CHECK_STATUS macro    */
main ()
{
   unsigned long          status;          /* error status                   */
   rpc_binding_vector_t   *binding_vector; /* binding handle list            */

   status =                       /* error status */
   RpcServerRegisterIf(           /* register interface with the RPC runtime */
      rfile_v1_0_s_ifspec,        /* handle for interface specification      */
      NULL,
      NULL
   );
   CHECK_STATUS(status, "Can't register interface", ABORT);

   status =
   RpcServerUseAllProtseqs(                /* establish protocol sequences  */
      RPC_C_PROTSEQ_MAX_REQS_DEFAULT,      /* queue length for remote calls */
      NULL                                 /* no security descriptor        */
   );
   CHECK_STATUS(status, "Can't establish protocol sequences", ABORT);

   status =
   RpcServerInqBindings(        /* get set of this server's binding handles */
      &binding_vector
   );
   CHECK_STATUS(status, "Can't get binding handles", ABORT);

   status =
   RpcEpRegister(                    /* add endpoint to local endpoint map  */
      rfile_v1_0_s_ifspec,           /* handle for interface specification  */
      binding_vector,                /* vector of server binding handles    */
      NULL,                          /* no object UUIDs to register         */
      (unsigned char *)"remote_file server"    /* annotation (not required) */
   );
   CHECK_STATUS(status, "Can't add endpoints to local endpoint map:", ABORT);

   puts("Listening for remote procedure calls...");
   RpcTryFinally
   {
      status =
      RpcServerListen(                         /* listen for remote calls   */
         1,                                    /* Minimum number of threads */
         RPC_C_LISTEN_MAX_CALLS_DEFAULT,       /* Maximum number of threads */
         NULL
      );
      CHECK_STATUS(status, "rpc listen failed:", RESUME);
   }
   RpcFinally
   {
      puts("Removing endpoints from local endpoint map.");
      status =
      RpcEpUnregister(          /* remove endpoints from local endpoint map */
         rfile_v1_0_s_ifspec,         /* handle for interface specificaiton */
         binding_vector,              /* vector of server binding handles   */
         NULL                         /* no object UUIDs to unregister      */
      );
      CHECK_STATUS(status,"Can't remove endpoints from endpoint map:", RESUME);

      status =
      RpcBindingVectorFree(                  /* free set of binding handles */
         &binding_vector
      );
      CHECK_STATUS(status, "Can't free binding handles and vector", ABORT);
   }
   RpcEndFinally
}

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );

}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}
